@@ -22,6 +22,7 @@
 #include "config.h"
 
 #include <math.h>
+#include <string.h>
 
 #include "pangocairo.h"
 #include "pangocairo-private.h"
@@ -206,6 +207,30 @@ _pango_cairo_font_install (PangoFont *font,
 }
 
 
+static int
+max_glyph_width (PangoLayout *layout)
+{
+  int max_width = 0;
+  GSList *l, *r;
+
+  for (l = pango_layout_get_lines_readonly (layout); l; l = l->next)
+    {
+      PangoLayoutLine *line = l->data;
+
+      for (r = line->runs; r; r = r->next)
+	{
+	  PangoGlyphString *glyphs = ((PangoGlyphItem *)r->data)->glyphs;
+	  int i;
+
+	  for (i = 0; i < glyphs->num_glyphs; i++)
+	    if (glyphs->glyphs[i].geometry.width > max_width)
+	      max_width = glyphs->glyphs[i].geometry.width;
+	}
+    }
+
+  return max_width;
+}
+
 typedef struct _PangoCairoFontMetricsInfo
 {
   const char       *sample_str;
@@ -239,6 +264,14 @@ _pango_cairo_font_get_metrics (PangoFont     *font,
       PangoFontMap *fontmap;
       PangoContext *context;
       cairo_font_options_t *font_options;
+      PangoLayout *layout;
+      PangoRectangle extents;
+      PangoFontDescription *desc;
+      cairo_scaled_font_t *scaled_font;
+      cairo_matrix_t cairo_matrix;
+      PangoMatrix pango_matrix;
+      PangoMatrix identity = PANGO_MATRIX_INIT;
+
       int height, shift;
 
       /* XXX this is racy.  need a ref'ing getter... */
@@ -253,14 +286,60 @@ _pango_cairo_font_get_metrics (PangoFont     *font,
 
       info->sample_str = sample_str;
 
+      scaled_font = _pango_cairo_font_private_get_scaled_font (cf_priv);
+
       context = pango_font_map_create_context (fontmap);
       pango_context_set_language (context, language);
+
       font_options = cairo_font_options_create ();
-      cairo_scaled_font_get_font_options (_pango_cairo_font_private_get_scaled_font (cf_priv), font_options);
+      cairo_scaled_font_get_font_options (scaled_font, font_options);
       pango_cairo_context_set_font_options (context, font_options);
       cairo_font_options_destroy (font_options);
 
-      info->metrics = (* PANGO_CAIRO_FONT_GET_IFACE (font)->create_metrics_for_context) (cfont, context);
+      info->metrics = (* PANGO_CAIRO_FONT_GET_IFACE (font)->create_base_metrics_for_context) (cfont, context);
+
+      /* We now need to adjust the base metrics for ctm */
+      cairo_scaled_font_get_ctm (scaled_font, &cairo_matrix);
+      pango_matrix.xx = cairo_matrix.xx;
+      pango_matrix.yx = cairo_matrix.yx;
+      pango_matrix.xy = cairo_matrix.xy;
+      pango_matrix.yy = cairo_matrix.yy;
+      pango_matrix.x0 = 0;
+      pango_matrix.y0 = 0;
+      if (G_UNLIKELY (0 != memcmp (&identity, &pango_matrix, 4 * sizeof (double))))
+        {
+	  double xscale = pango_matrix_get_font_scale_factor (&pango_matrix);
+	  if (xscale) xscale = 1 / xscale;
+
+	  info->metrics->ascent *= xscale;
+	  info->metrics->descent *= xscale;
+	  info->metrics->underline_position *= xscale;
+	  info->metrics->underline_thickness *= xscale;
+	  info->metrics->strikethrough_position *= xscale;
+	  info->metrics->strikethrough_thickness *= xscale;
+	}
+
+
+      /* Set the matrix on the context so we don't have to adjust the derived
+       * metrics. */
+      pango_context_set_matrix (context, &pango_matrix);
+
+      /* Update approximate_*_width now */
+      layout = pango_layout_new (context);
+      desc = pango_font_describe_with_absolute_size (font);
+      pango_layout_set_font_description (layout, desc);
+      pango_font_description_free (desc);
+
+      pango_layout_set_text (layout, sample_str, -1);
+      pango_layout_get_extents (layout, NULL, &extents);
+
+      info->metrics->approximate_char_width = extents.width / pango_utf8_strwidth (sample_str);
+
+      pango_layout_set_text (layout, "0123456789", -1);
+      info->metrics->approximate_digit_width = max_glyph_width (layout);
+
+      g_object_unref (layout);
+
 
       /* We may actually reuse ascent/descent we got from cairo here.  that's
        * in cf_priv->font_extents.
