@@ -102,9 +102,9 @@ pango_ft2_font_map_finalize (GObject *object)
   if (ft2fontmap->substitute_destroy)
     ft2fontmap->substitute_destroy (ft2fontmap->substitute_data);
 
-  FT_Done_FreeType (ft2fontmap->library);
-
   G_OBJECT_CLASS (pango_ft2_font_map_parent_class)->finalize (object);
+
+  FT_Done_FreeType (ft2fontmap->library);
 }
 
 /**
@@ -341,7 +341,7 @@ _pango_ft2_font_map_default_substitute (PangoFcFontMap *fcfontmap,
 
 static double
 pango_ft2_font_map_get_resolution (PangoFcFontMap       *fcfontmap,
-				   PangoContext         *context)
+				   PangoContext         *context G_GNUC_UNUSED)
 {
   return ((PangoFT2FontMap *)fcfontmap)->dpi_y;
 }
